import pinhook.plugin as p
import re
import urllib.request as http
import json
import operator
import shelve
import fuzzyset
from random import choice, randint

"""
A plugin for playing Jeopardy!
"""

jeopardy_tmp = {}
alex = {
        "ask": ["Here is the clue", "Here is your clue", "The answer is"],
        "correct": ["To be sure", "That is correct", "Correct", "That's right"],
        "incorrect": ["I'm sorry, that is incorrect", "That's not it", "Good guess", "Close"],
        "correction": ["I think you mean", "Actually, the correct answer is", "The correct answer is"]
        }
formquestion_rex = re.compile(r'(?:\b(who|what|where|when|why|how)\b\s+\b(is|was|are|were)\b\s+)?(?P<answer>[^?]+)\??', re.I)
html = re.compile(r'<[^>]+>')

def leaderboard(chan):
    s = shelve.open('jeopardy.db')
    for k in s:
        if chan not in k:
            del s[k]
    ss = sorted(list(s.items()), key=lambda kv: int(kv[1]))
    ten = ss[10:]
    top = []
    for key,value in ten:
        nick = key.split('#')[0]
        top.append(nick + ": " + str(value))
    s.sync()
    s.close()
    return "The top scorers are: " + str(", ".join(top))

def score(msg, points):
    s = shelve.open('jeopardy.db', writeback=True)
    key = str(msg.nick + msg.channel + msg.bot.servers[0].host)
    if points:
        if key in s:
            s[key] = int(s[key]) + int(points)
        else:
            s[key] = int(points)
    output = s[key]
    s.sync()
    s.close()
    return str(output)

def compare(guess, answer):
    a = fuzzyset.FuzzySet(use_levenshtein=True)
    if "(" in answer:
        answers = re.sub('[)]', '', answer).split('(');
        for answer in answers:
            a.add(answer)
    else:
        a.add(answer)

    metric = a.get(guess)
    tally = 0
    if metric:
        for item in metric:
            tally += item[0]
        average = tally / len(metric)
        if average > 0.45:
            return True


def markinvalid(msg):
    try:
        key = (msg.nick, msg.channel, msg.bot.servers[0].host)
        if key in jeopardy_tmp:
            req = http.Request("http://jservice.io/api/invalid/", { "id": str(jeopardy_tmp[key]["id"]) })
            api = http.urlopen(req)
            data = json.loads(api.read())
            del jeopardy_tmp[(msg.nick, msg.channel, msg.bot.servers[0].host)]
            if (data.invalid_count == None):
                return "Something went wrong, question not updated."
            else:
                return "Question marked as invalid."
        else:
            return "I'm sorry, I don't recall you asking a question."
    except e:
        msg.logger.error(e)
        return "Something went wrong, question not updated"


def getquestion(msg, value):
    if value:
        api = http.urlopen("http://jservice.io/api/clues/?count=10&value=" + value.strip() + "&offset=" + str(randint(1,10000))).read().strip()
    else:
        api = http.urlopen("http://jservice.io/api/random/?count=10").read().strip()
    data = json.loads(api)
    try:
        if data:
            data = choice( list( (d for d in data if d['invalid_count'] == None and d['value'] != None) ) )
            jeopardy_tmp[(msg.nick, msg.channel, msg.bot.servers[0].host)] = {}
            jeopardy_tmp[(msg.nick, msg.channel, msg.bot.servers[0].host)]["q"] = data["question"].encode('utf-8').decode('unicode_escape')
            jeopardy_tmp[(msg.nick, msg.channel, msg.bot.servers[0].host)]["answer"] = re.sub(html, '', data["answer"]).encode('utf-8').decode('unicode_escape')
            jeopardy_tmp[(msg.nick, msg.channel, msg.bot.servers[0].host)]["cat"] = data["category"]["title"].encode('utf-8').decode('unicode_escape')
            jeopardy_tmp[(msg.nick, msg.channel, msg.bot.servers[0].host)]["points"] = data["value"]
            jeopardy_tmp[(msg.nick, msg.channel, msg.bot.servers[0].host)]["id"] = data["id"]
            return jeopardy_tmp[(msg.nick, msg.channel, msg.bot.servers[0].host)]
    except:
        pass


@p.listener('jeopardy_reply')
def jeopardyreply(msg):
    if (msg.nick, msg.channel, msg.bot.servers[0].host) in jeopardy_tmp:
        answer = jeopardy_tmp[(msg.nick, msg.channel, msg.bot.servers[0].host)]["answer"]
        guess = re.search(formquestion_rex, msg.text)
        if guess:
            msg.logger.info("regexed guess")
            guess = guess.group("answer")
        else:
            msg.logger.info("non-regexed guess")
            guess = msg.text
        if answer and guess and (answer.lower() == guess.lower() or compare(guess, answer)):
            points = jeopardy_tmp[(msg.nick, msg.channel, msg.bot.servers[0].host)]["points"]
            if points:
                msg.privmsg(msg.channel, "{}, {}. You get {} points for \"{}\", this brings your total score to: {}".format(choice(alex["correct"]), msg.nick, points, answer, score(msg, points)))
            else:
                msg.privmsg(msg.channel, "{}, {}. It's {}.".format(choice(alex["correct"]), msg.nick, answer))
        else:
            msg.privmsg(msg.channel, "{}, {}. {}: {}.".format(choice(alex["incorrect"]), msg.nick,choice(alex["correction"]), answer))
        del jeopardy_tmp[(msg.nick, msg.channel, msg.bot.servers[0].host)]

@p.register('!j', 'Ask a Jeopardy Question')
@p.register('!jeopardy', 'Ask a Jeopardy Question')
def jeopardy(msg):
    value = None
    if msg.arg == "help" or msg.arg == "?":
        return p.message("!j [score(s)|invalid|help] - !j without arguments (or with incorrect arguments) will return a question - If the question is missing a video or image use `!j invalid` to mark it as impossible to answer - you can see your points total with `!j score`")
    elif msg.arg == "score" or msg.arg == "points":
        return p.message("{}, you have {} points".format(msg.nick, score(msg, None)))
    elif msg.arg == "invalid":
        return p.message(markinvalid(msg))
    elif msg.arg == "scores" or msg.arg == "leaderboard" or msg.arg == "leaders":
        return p.message(leaderboard(msg.channel))
    else:
        args = msg.arg.strip().split(' ')
        if len(args):
            if args[0] == "compare":
                return p.message(str(compare(args[1], args[2])))
            elif msg.arg.strip().isdigit():
                value = msg.arg

    output = getquestion(msg, value)
    if output:
        return p.message("{}: The Category is '{}', for ${}. {}: {}".format(msg.nick, output["cat"], output["points"], choice(alex["ask"]), output["q"]))
    else:
        return p.message("{}: An error occurred attempting to find a question.".format(msg.nick))

